/*****************************************************************************
 * dr.h
 * Copyright (C) 2001-2010 VideoLAN
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *****************************************************************************/

/*!
 * \file <dr.h>
 * \author Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 * \brief Gather all dr_*.h into one.
 *
 * Gathers all dr_*.h into one. Use this header if you need a lot of them.
 */

#ifndef _DVBPSI_DR_H_
#define _DVBPSI_DR_H_

#include "dr_02.h"
#include "dr_03.h"
#include "dr_04.h"
#include "dr_05.h"
#include "dr_06.h"
#include "dr_07.h"
#include "dr_08.h"
#include "dr_09.h"
#include "dr_0a.h"
#include "dr_0b.h"
#include "dr_0c.h"
#include "dr_0d.h"
#include "dr_0e.h"
#include "dr_0f.h"
#include "dr_10.h"
#include "dr_11.h"
#include "dr_12.h"
#include "dr_13.h"
#include "dr_14.h"
#include "dr_1b.h"
#include "dr_1c.h"
#include "dr_40.h"
#include "dr_41.h"
#include "dr_42.h"
#include "dr_43.h"
#include "dr_44.h"
#include "dr_45.h"
#include "dr_47.h"
#include "dr_48.h"
#include "dr_49.h"
#include "dr_4a.h"
#include "dr_4b.h"
#include "dr_4c.h"
#include "dr_4d.h"
#include "dr_4e.h"
#include "dr_4f.h"
#include "dr_50.h"
#include "dr_52.h"
#include "dr_53.h"
#include "dr_54.h"
#include "dr_55.h"
#include "dr_56.h"
#include "dr_58.h"
#include "dr_59.h"
#include "dr_5a.h"
#include "dr_62.h"
#include "dr_66.h"
#include "dr_69.h"
#include "dr_73.h"
#include "dr_76.h"
#include "dr_7c.h"
#include "dr_81.h"
#include "dr_83.h"
#include "dr_86.h"
#include "dr_8a.h"
#include "dr_a0.h"
#include "dr_a1.h"

#else
#error "Multiple inclusions of dr.h"
#endif
